#!/usr/bin/env sh
# shellcheck disable=SC1073

onedatify_version="2"

onedatify_config_dir="./"
onedatify_config_dir="/opt/onedata/onedatify"
config_file_path="${onedatify_config_dir}/config"
template_config_file_path="${onedatify_config_dir}/config.template"
onedatify_docker_compose_template_path="${onedatify_config_dir}/docker-compose.yml.template"
onedatify_docker_compose_path="${onedatify_config_dir}/docker-compose.yml"
local_copy_of_docker_compose_yaml=$(cat $onedatify_docker_compose_template_path)
rest_cli_docker_image=onedata/rest-cli:21.02.0-alpha13

# corss platform compatibility
aliases() {
  case $( uname -s ) in
    Linux)
          _expr=expr
          ;;
    Darwin)
          _expr=gexpr
          ;;
  esac

  curl=''
  if command_exists curl; then
    curl='curl -sSL'
  elif command_exists wget; then
    curl='wget -qO-'
  fi
}

command_exists() {
  command -v "$@" > /dev/null 2>&1
}

###############################################################
# Utility functions that will be called only once, script wide
###############################################################
get_sh() {
  user="$(id -un 2>/dev/null || true)"
  sh_c='sh -c'
  if [ "$user" != 'root' ]; then
    if command_exists sudo; then
      sh_c='sudo -E sh -c'
    elif command_exists su; then
      sh_c='su -c'
    else
cat >&2 <<EOF
Error: this installer needs the ability to run commands as root.
Neither the 'sudo' nor 'su' command was found on the host, interrupting.
EOF
exit 1
    fi
  fi
}
# Global shell variable used by all functions
get_sh #1>/dev/null

#######################################
# Utility logging and colour functions
#######################################
colors() {
  if [ -t 1 ] ; then
    def_color=$(tput sgr0 || tput me )
    red=$(tput setaf 1; tput bold || tput md)
    green=$(tput setaf 2;tput bold || tput md)
    yellow=$(tput setaf 3; tput bold || tput md)
    blue=$(tput setaf 4; tput bold || tput md)
    magneta=$(tput setaf 5; tput bold || tput md)
    cyan=$(tput setaf 6; tput bold || tput md)
    grey=$(tput setaf 7;)
    grey_bold=$(tput setaf 7 ; tput bold || tput md )
  else
    def_color=''
    red=''
    green=''
    yellow=''
    blue=''
    magneta=''
    cyan=''
    grey=''
    grey_bold=''
  fi
}
colors #1>/dev/null

log() {
  printf '%s%s%s%s\n' "$1" "$2" "$3" "$def_color" >&2
}
_log() {
  printf '%s%s%s%s' "$1" "$2" "$3" "$def_color" >&2
}

message() {
  log "$grey_bold" "" "$@"
}
message_imp() {
  log "$red" "" "$@"
}

_message_imp() {
  _log "$red" "" "$@"
}
_message() {
  _log "$grey_bold" "" "$@"
}

################
# Usage message
################
usage() {
cat <<EEOF
onedatify: easily bootstrap a single node Oneprovider cluster.

    ┌─────────────────────────────────────────────────────────────┐
    │ ONEDATIFY IS CURRENTLY IN BETA                              │
    │                                                             │
    │ But please, try it out and give us feedback at:             │
    │ https://github.com/onedata/onedatify                        │
    └─────────────────────────────────────────────────────────────┘

Example usage:

    Create a single node Oneprovider cluster which joins the
    https://onezone.onedata.org zone, supports a space and imports
    data from the /tmp/ directory.

    ┌─────────────────────────────────────────────────────────────┐
    │ 1) Install the Oneprovider on this machine and connect      │
    │    to the Onezone.                                          │
    ├─────────────────────────────────────────────────────────────┤
    │ onedatify install --onezone-url https://onezone.onedata.org │
    └─────────────────────────────────────────────────────────────┘

    ┌─────────────────────────────────────────────────────────────┐
    │ 2) Get a space support token from the Onezone.              │
    ├─────────────────────────────────────────────────────────────┤
    │ Example token: MDAXNWxvY2F00aW9uIG9uZXpvbmUKMDAzYmlkZW50    │
    │ 0aWZpZXIgOUgwMFhtOW4ta1ZVLWFkdEZCanNLNjlmNnJQeHUwSkJGUDk    │
    │ ydU0000VFZEVQowMDI4Y2lkIHRva2VuVHlwZSA9IHNwYWNlX3N1cHBvc    │
    │ nRfdG9rZW4KMDAyZnNpZ25hdHVyZSCNt01pHhY9Rtwp80100eDbctCLm    │
    │ nwhm7GtFpRauhFKogDIwo                                       │
    └─────────────────────────────────────────────────────────────┘

    ┌─────────────────────────────────────────────────────────────┐
    │ 3) Use a token to support a space and import data from      │
    │    the POSIX storage backend:                               │
    ├─────────────────────────────────────────────────────────────┤
    │ onedatify support posix --token MDAXNWxvY2F00aW9ubm...      │
    │                         --import-directory /tmp             │
    └─────────────────────────────────────────────────────────────┘

    You can then repeat the second and the third step to support
    as many spaces as you like using different kinds of storage backends.

Usage:
  onedatify [command] [ --help ]

Lifecycle Management Commands:
  start       start the Oneprovider
  stop        stop the Oneprovider
  restart     restart the Oneprovider
  upgrade     upgrade the Oneprovider to the new version
  install     set up the Oneprovider

Configuration Management Commands:
  config      configure the Oneprovider
  storage     add a storage backend to the Oneprovider
  support     support a space with a storage backend and expose your existing data as a space

Troubleshooting and Debugging Commands:
  console     start rest-cli console to manage and query the Oneprovider
  logs        acquire Oneprovider logs.

Other Commands:
  version     display version of Onedatify, Oneprovider, and Onezone


Use "onedatify [command] --help" for more information about the command

EEOF
}

main() {

  if [ $(( $# )) -eq 0 ]; then
    usage
    exit 0
  fi

  while [ $(( $# )) -ne 0 ]; do
      case $1 in
          -h|-\?|--help)   # Call a "usage" function to display a synopsis, then exit.
              usage
              exit 0
              ;;
          config)
              shift
              # shellcheck source=onedatify/config.sh
              . "${onedatify_config_dir}/config.sh"
              config_main "$@"
              exit $?
              ;;
          console)
              shift
              # shellcheck source=onedatify/console.sh
              . "${onedatify_config_dir}/console.sh"
              console_main "$@"
              exit $?
              ;;
          install)
              shift
              # shellcheck source=onedatify/install.sh
              . "${onedatify_config_dir}/install.sh"
              install_main "$@"
              exit $?
              ;;
          logs)
              shift
              # shellcheck source=onedatify/logs.sh
              . "${onedatify_config_dir}/logs.sh"
              logs_main "$@"
              exit $?
              ;;
          restart)
              shift
              onedatify_restart
              exit $?
              ;;
          start)
              shift
              onedatify_start
              exit $?
              ;;
          stop)
              shift
              onedatify_stop
              exit $?
              ;;
          storage)
              shift
              # shellcheck source=onedatify/storage.sh
              . "${onedatify_config_dir}/storage.sh"
              storage_main "$@"
              exit $?
              ;;
          support)
              shift
              # shellcheck source=onedatify/support.sh
              . "${onedatify_config_dir}/support.sh"
              support_main "$@"
              exit $?
              ;;
          upgrade)
              shift
              # shellcheck source=onedatify/upgrade.sh
              . "${onedatify_config_dir}/upgrade.sh"
              upgrade_main "$@"
              exit $?
              ;;
          version)
              shift
              version
              exit $?
              ;;
          -?*|*)
              printf 'WARN: Unknown option (ignored): %s\n' "$1" >&2
              exit 1
              ;;
      esac
      shift
  done

}


onedatify_start() {
  $sh_c "systemctl enable --quiet onedatify.service"
  $sh_c "systemctl start onedatify.service"
}

onedatify_stop() {
  $sh_c "docker exec onedatify-oneprovider-1 /root/oneprovider-stop-graceful.sh"
  $sh_c "systemctl disable --quiet onedatify.service"
  $sh_c "systemctl stop onedatify.service"
}

onedatify_restart() {
  $sh_c "systemctl restart onedatify.service"
}

onedatify_is_active() {
  $sh_c "systemctl is-active --quiet onedatify"
}

onedatify_status() {
  $sh_c "systemctl status onedatify"
}

version() {
  # shellcheck source=onedatify/config.sh
  . "${onedatify_config_dir}/config.sh"
  if ! config_main validate 1; then
    echo "ERROR. Configuration is not complete or contains errors. Please run onedatify config to fix it"
    exit 1
  fi
  echo "Onedatify version: $onedatify_version"
  echo "Oneprovider docker image: $config_oneprovider_version"
  version_oneprovider_api=$(curl -s -q "https://localhost/version")
  [ "$version_oneprovider_api" = "" ] && version_oneprovider_api="error getting version, is the Oneprovider running?"
  echo "Oneprovider api version: $version_oneprovider_api"
  version_onezone_api=$(curl -s -q "https://$config_zone_fqdn/version")
  [ "$version_onezone_api" = "" ] && version_onezone_api="error getting version, is the Oneprovider running?"
  echo "Onezone api version: $version_onezone_api"
}

aliases
main "$@"
