#!/usr/bin/env sh
# shellcheck source=onedatify/onedatify


storage_usage() {
cat <<EOF
Add a storage backend to a Oneprovider or get information about storage backends registered with the Oneprovider.

onedatify storage [ get | add ] [ --help ]

Commands:
  get       get information about storage backends connected to oneprovider
  add       connect a storage backend to the oneprovider

Each command provides its own documentation, use --help.
EOF
}

storage_main() {

  if [ $(( $# )) -eq 0 ]; then
    storage_usage
    exit 0
  fi

  . "${onedatify_config_dir}/config.sh"
  if ! config_main validate 1; then
    echo "ERROR. Configuration is not complete or contains errors. Please run onedatify config to fix it"
    exit 1
  fi

  while [ $(( $# )) -ne 0 ] ; do
      case $1 in
          -h|-\?|--help)
              storage_usage
              exit 0
              ;;
          add)
              shift
              storage_add_main "$@"
              exit $?
              ;;
          get)
              shift
              storage_get "$@"
              exit $?
              ;;
          interactive)
              shift
              storage_interactive_flag=1
              storage_interactive "$@"
              exit $?
              ;;
          interactive-import)
              shift
              storage_interactive_flag=1
              storage_interactive_ask_import_questions_flag=1
              storage_interactive "$@"
              exit $?
              ;;
          -?*|*)
              printf 'WARN: Unknown option (ignored): %s\n' "$1" >&2
              exit 1
              ;;
      esac
      shift
  done
}

#######################################
# storage list section
#
#######################################
storage_usage_get() {
cat <<EOF
onedatify storage get [ --selector <key=value,key1=value1,...> ] [ --fields <f1,f2,...> ] [ -v ]

Examples:
Get information about all storage backends registered with the Oneprovider:
onedatify storage get

Get only storage backends named "fast storage":
onedatify storage get -s name="fast storage"

Get only storage backends named "fast storage" of type "ceph":
onedatify storage get -s name="fast storage",type=ceph

Query only the fields 'name' and 'type' of storage backends named "fast storage" of type "ceph":
onedatify storage get -s name="fast storage",type=ceph -f name,type

Query only the fields 'name' and 'type' of storage backends named "fast storage" of type "ceph"
and display only values of those fields:
onedatify storage get -s name="fast storage",type=ceph -f name,type -v
EOF
}

storage_get() {

  while [ $(( $# )) -ne 0 ] ; do
      case $1 in
          -h|-\?|--help)
              storage_usage_get
              exit 0
              ;;
          -s|--selector)
              storage_get_selector="$2"
              # TODO when empty argument
              # /usr/bin/onedatify: 97: shift: can't shift that many
              shift
              ;;
          -f|--fields)
              storage_get_fields="$2"
              # TODO when empty argument
              # /usr/bin/onedatify: 97: shift: can't shift that many
              shift
              ;;
          -v|--values-only)
              storage_get_values_only=1
              ;;
          -?*|*)
              printf 'storage_get WARN: Unknown option (ignored): %s\n' "$1" >&2
              exit 1
              ;;
      esac
      shift
  done

  storage_get_selectors=$(echo "$storage_get_selector" | sed -e 's/\([^\/]\),/\1\n/g' )
  storage_get_select_jq='select('
  for f in $storage_get_selectors; do
    storage_get_select_key=${f%%=*}
    storage_get_select_key_tmp=${f#$storage_get_select_key}
    storage_get_select_val=${storage_get_select_key_tmp#=}
    storage_get_select_jq="$storage_get_select_jq(.${storage_get_select_key}==\"${storage_get_select_val}\") and "
  done
  storage_get_select_jq="${storage_get_select_jq% and }"
  storage_get_select_jq="${storage_get_select_jq})"

  if [ "$storage_get_select_jq" = "select()" ]; then
    storage_get_select_jq=""
  else
    storage_get_select_jq="| $storage_get_select_jq"
  fi


# jq '{"id":(.id),"name":(.name)}'

  storage_get_fields_sep=$(echo "$storage_get_fields" | sed -e 's/\([^\/]\),/\1\n/g' )
  storage_get_feild_jq="{"
  for f in $storage_get_fields_sep; do
    storage_get_feild_jq="$storage_get_feild_jq\"$f\":(.${f}), "
  done
  storage_get_feild_jq="${storage_get_feild_jq%, }"
  storage_get_feild_jq="${storage_get_feild_jq}}"

  if [ "$storage_get_feild_jq" = "{}" ]; then
    storage_get_feild_jq=""
  else
    storage_get_feild_jq="| $storage_get_feild_jq"
  fi

  if [ "$storage_get_values_only" = "1" ]; then
    storage_get_values_only_jq=" | .[] "
  else
    storage_get_values_only_jq=""
  fi

get_storage_script=$(cat <<EEOF
#!/usr/bin/env bash
cat <<EOF >/tmp/json
. $storage_get_select_jq $storage_get_feild_jq $storage_get_values_only_jq
EOF
onepanel-rest-cli getStorages | jq  -r ".ids[]" | xargs -I{} onepanel-rest-cli getStorageDetails id={}  | jq -r -f /tmp/json
EEOF
  )

echo "$get_storage_script" | $sh_c "docker run \
    -i \
    --network=host \
    -e TERM=xterm \
    -e ONEPANEL_HOST=\"https://localhost:9443\" \
    -e ONEPROVIDER_HOST=\"https://localhost\" \
    -e ONEPANEL_BASIC_AUTH=\"onepanel:$config_admin_password\" \
    --rm \
    --entrypoint bash \
    \"$rest_cli_docker_image\" \
    -c \
    'echo \"-k\" > ~/.curlrc ; cat > script.sh ; chmod +x script.sh ; ./script.sh'"
}

#######################################
# storage add section
#
#######################################
storage_add_usage() {
cat <<EOF
onedatify storage add [ posix | s3 | ceph | gluster | swift ] 

  Options common to all the storage backends:
  -n, --name              the name of storage
  -r, --read-only         whether storage is readonly (default: fase)
  -i, --insecure          whether storage administrator credentials (username and key) may be used by users
  -x, --imported-storage  import the content of the storage to the space
                          without storage accounts to access storage in direct IO mode. (default: false)
  -t, --timeout           storage operation timeout in milliseconds (default: 5 min)
  --luma-url              URL of external LUMA service

  posix
    -m, --mount-point     the absolute path to the directory where the POSIX storage is mounted on the host machine

  s3
    -n, --hostname            hostname of a machine where S3 storage is installed
    -b, --bucket-name         storage bucket name
    -a, --access-key          access key to the S3 storage
    -s, --secret-key          secret key to the S3 storage
    -b, --block-size          storage block size in bytes
    -v, --signature-version   version of signature used to sign requests
                              possible values are: 2 and 4 (default: 4)

  ceph
    -n, --cluster-name        ceph cluster name
    -u, --username            username of the Ceph cluster administrator
    -k, --key                 admin key to access the Ceph cluster
    -m, --monitor-hostname    ceph monitor host name
    -p, --pool-name           ceph pool name

  gluster
    -n, --hostname            hostname (IP address or FQDN) of GlusterFS volume server
    -v, --volume              name of the volume to use as a storage backend.
    -p, --port                GlusterFS port on volume server (default: 24007)
    -t, --transport-protocol  transport protocol to use to connect to the volume server
                              valid values: tcp, rdma, and socket (default: tcp)
    -m, --mount-point         relative mountpoint within the volume which should be used by Oneprovider (default: "")
    -x, --xlator-options      volume specific GlusterFS translator options in the format:
                              TRANSLATOR1.OPTION1=VALUE1;TRANSLATOR2.OPTION2=VALUE2; (default: "")

  swift
    -a, --auth-url            URL to OpenStack Keystone identity service
    -t, --tenant-nam          name of the tenant to which the user belongs
    -c, --container-name      name of the Swift storage container
    -u, --username            Keystone authentication username
    -v, --password            Keystone authentication password
    -b, --block-size          storage block size in bytes
EOF
}

storage_add_main() {

  if [ $(( $# )) -eq 0 ]; then
    storage_add_usage
    exit 0
  fi

  storage_add_name=""
  storage_add_type=""

  # optional
  storage_add_read_only=0
  storage_add_insecure=0
  storage_add_imported_storage=0
  storage_add_luma_url=""
  storage_add_timeout=""

  case $1 in
      -h|-\?|--help)   # Call a "usage" function to display a synopsis, then exit.
          storage_add_usage
          exit 0
          ;;
      posix|ceph|gluster|s3|swift)
          storage_add_type="$1"
          ;;
      -?*|*)
          printf 'WARN: Unsupported storage type!: %s\n' "$1" >&2
          exit 1
          ;;
  esac
  shift

  storage_add_main_storage_specific_options=""
  while [ $(( $# )) -ne 0 ] ; do
      case $1 in
          -n|--name)
              storage_add_name="$2"
              shift
              ;;
          -r|--read-only)
              storage_add_read_only=1
              ;;
          -i|--insecure)
              storage_add_insecure=1
              ;;
          -x| --imported-storage)
              storage_add_imported_storage=1
              ;;
          -t|--timeout)
              storage_add_timeout="$2"
              shift
              ;;
          --luma-url)
              storage_add_luma_url="$2"
              shift
            ;;
          *)
          storage_add_main_storage_specific_options="$storage_add_main_storage_specific_options $1"
      esac
      shift
  done
  storage_add_main_storage_specific_options="${storage_add_main_storage_specific_options# }"

  storage_add_common_json="$(storage_add_common_json_create $storage_add_read_only $storage_add_imported_storage $storage_add_insecure $storage_add_timeout)"
  storage_add_luma_json=$(storage_add_luma_create "$storage_add_luma_url")

  case $storage_add_type in
      posix)
          [ "$storage_add_name" = "" ] && storage_add_name=$storage_add_type
          # shellcheck disable=SC2086
          storage_add_posix_main "$storage_add_name" "$storage_add_common_json" "$storage_add_luma_json" $storage_add_main_storage_specific_options
          exit $?
          ;;
      ceph)
          [ "$storage_add_name" = "" ] &&  storage_add_name=$storage_add_type
          # shellcheck disable=SC2086
          storage_add_ceph_main "$storage_add_name" "$storage_add_common_json" "$storage_add_luma_json" $storage_add_main_storage_specific_options
          exit $?
          ;;
      gluster)
          [ "$storage_add_name" = "" ] &&  storage_add_name=$storage_add_type
          # shellcheck disable=SC2086
          storage_add_gluster_main "$storage_add_name" "$storage_add_common_json" "$storage_add_luma_json" $storage_add_main_storage_specific_options
          exit $?
          ;;
      s3)
          [ "$storage_add_name" = "" ] &&  storage_add_name=$storage_add_type
          # shellcheck disable=SC2086
          storage_add_s3_main "$storage_add_name" "$storage_add_common_json" "$storage_add_luma_json" $storage_add_main_storage_specific_options
          exit $?
          ;;
      swift)
          shift
          [ "$storage_add_name" = "" ] &&  storage_add_name=$storage_add_type
          # shellcheck disable=SC2086
          storage_add_swift_main "$storage_add_name" "$storage_add_common_json" "$storage_add_luma_json" $storage_add_main_storage_specific_options
          exit $?
          ;;
      -?*|*)
          printf 'WARN: Unsupported storage type!: %s\n' "$1" >&2
          exit 1
          ;;
  esac
}

#
# storage add
# utility functions
#
storage_add_common_json_create() {
  if [ "$1" -eq "1" ]; then
    storage_add_read_only_create="true"
  else
    storage_add_read_only_create="false"
  fi

  if [ "$2" -eq "1" ]; then
    storage_add_imported_storage_create="true"
  else
    storage_add_imported_storage_create="false"
  fi

  if [ "$3" -eq "1" ]; then
    storage_add_insecure_create="true"
  else
    storage_add_insecure_create="false"
  fi

  cat <<EOF
     ,"readonly": "$storage_add_read_only_create",
      "importedStorage": "$storage_add_imported_storage_create",
      "insecure": "$storage_add_insecure_create"
EOF
}

storage_add_luma_create() {
  if [ "$1" != "" ]; then
    storage_add_lumaEnabled_create="true"
    # TODO validate luma url
    storage_add_lumaUrl_create="$1"

    cat <<EOF
    ,"lumaEnabled": "$storage_add_lumaEnabled_create",
      "lumaUrl": "$storage_add_lumaUrl_create"
EOF
  else
    echo ""
  fi

# TODO add all luma options
# storage_add_luma_json=$(cat <<EOF
#      ,"lumaEnabled": "$storage_add_lumaEnabled",
#       "lumaUrl": "$storage_add_lumaUrl",
#       "lumaCacheTimeout": "$storage_add_lumaCacheTimeout",
#       "lumaApiKey": "$storage_add_lumaApiKey"
# EOF
# )
}

storage_add_ask_for_restart() {
  while : ; do
    _message "Do you wish to continue with the restart (y/n)? " 
    read -r storage_add_ask_for_restart_ack </dev/tty
    if [ "$config_lets_encrypt_enabled" != "n" ] || [ "$config_lets_encrypt_enabled" != "y" ]; then
      break
    fi
  done
  if [ "$storage_add_ask_for_restart_ack" = "y" ]; then
    return 0
  else
    return 1
  fi
}

storage_interactive() {

  # There are two usecases of this script:
  # 1. setting up a provider backed by a storage 
  # 2. exposing data on that storage
  # all this needs to handle different types of storage backends

  ######
  # Ask for storage type
  ######
  message ''
  message '########### Storage Configuration #############'
  if [ "$storage_add_type_default" = "" ]; then
    storage_add_type_default="posix"
  else
    storage_add_type_default=$storage_add_type
  fi
  storage_add_type=""
  while true ; do
    _message "What storage type would you like to use? (default: $storage_add_type_default, possible options: posix | s3 | ceph | glusterfs | swift ): "
    read -r storage_add_type </dev/tty
    [ "$storage_add_type" = "" ] && storage_add_type=$storage_add_type_default
    case "$storage_add_type" in
      posix|s3|ceph|glusterfs|swift)
          break
          ;;
      *)
        message_imp "Possible storage types are: posix | s3 | ceph | glusterfs | swift."
    esac
  done

  # In this wizard we assume that all storage backends will be named the same
  storage_add_storage_name="onedatify"

#          --insecure "$storage_add_insecure" \
  case $storage_add_type in
      posix)
        storage_interactive_ask_posix
        storage_interactive_add() {
        storage_main add posix -n "$storage_add_storage_name" $1 \
          -m "$storage_add_posix_mountPoint"
        }
        ;;
      ceph)
        storage_interactive_ask_ceph
        storage_interactive_add() {
        storage_main add ceph -n "$storage_add_storage_name" $1 \
          --cluster-name "$storage_add_ceph_clusterName" \
          --username "$storage_add_ceph_username" \
          --key "$storage_add_ceph_key" \
          --monitor-hostname "$storage_add_ceph_monitorHostname" \
          --pool-name "$storage_add_ceph_poolName"
          #--block-size "$storage_add_ceph_blockSize"
        }
        ;;
      gluster)
        storage_interactive_ask_gluster
        storage_interactive_add() {
        storage_main add gluster -n "$storage_add_storage_name" $1 \
          --hostname "$storage_add_gluster_hostname" \
          --volume "$storage_add_gluster_volume" \
          --transport-protocol "$storage_add_gluster_transport"
          #--port "$storage_add_gluster_port" \
          #--mount-point "$storage_add_gluster_mountPoint" \
          #--xlator-options "$storage_add_gluster_xlatorOptions"
        }
        ;;
      s3)
        storage_interactive_ask_s3
        storage_interactive_add() {
        storage_main add s3 -n "$storage_add_storage_name" $1 \
          --hostname "$storage_add_s3_hostname" \
          --bucket-name "$storage_add_s3_bucketName" \
          --access-key "$storage_add_s3_accessKey" \
          --secret-key "$storage_add_s3_secretKey"
          #--signature-version "$storage_add_s3_signatureVersion"
        }
        ;;
      swift)
        storage_interactive_ask_swift
        storage_interactive_add() {
        storage_main add swift -n "$storage_add_storage_name" $1 \
          --read-only \
          --auth-url "$storage_add_swift_authUrl" \
          --tenant-name "$storage_add_swift_tenantName" \
          --container-name "$storage_add_swift_containerName" \
          --username "$storage_add_swift_username" \
          --password "$storage_add_swift_password"
          #--block-size "$storage_add_swift_blockSize"
        }
        ;;
      -?*|*)
        printf 'WARN: Unsupported storage type!: %s\n' "$1" >&2
        exit 1
        ;;
  esac
  storage_interactive_add_flags=""

  if [ "$storage_interactive_ask_import_questions_flag" = "1" ]; then
    storage_add_imported_storage=1 # No need to ask, it's import anyway
    storage_interactive_ask_luma_url
    storage_interactive_ask_storage_add_read_only
  fi

  # If the url is provider the storage is secure if not (luma disabled) make it insecure
  [ "$storage_add_luma_url" = "" ] && storage_interactive_add_flags="$storage_interactive_add_flags --insecure"

  [ "$storage_add_luma_url" != "" ] && storage_interactive_add_flags="$storage_interactive_add_flags --luma-url $storage_add_luma_url"
  [ "$storage_add_imported_storage" = 1 ] && storage_interactive_add_flags="$storage_interactive_add_flags --imported-storage"
  [ "$storage_add_read_only" = true ] && storage_interactive_add_flags="$storage_interactive_add_flags --read-only"

  storage_interactive_add "$storage_interactive_add_flags"
}

storage_interactive_ask_posix() {
  ######
  # Ask which directory to expose
  ######
  if [ "$storage_add_posix_mountPoint" = "" ]; then
    storage_add_posix_mountPoint_default="/tmp"
  else
    storage_add_posix_mountPoint_default=$storage_add_posix_mountPoint
  fi
  storage_add_posix_mountPoint=""
  while true ; do
    if [ "$storage_interactive_ask_import_questions_flag" = 1 ]; then
      _message "The absolute path to the directory you want to expose through the Onedata space (default: $storage_add_posix_mountPoint_default): "
    else
      _message "The absolute path to the directory where the space data will be stored (default: $storage_add_posix_mountPoint_default): "
    fi
    read -r storage_add_posix_mountPoint </dev/tty
    [ "$storage_add_posix_mountPoint" = "" ] && storage_add_posix_mountPoint=$storage_add_posix_mountPoint_default
    if [ -d "$storage_add_posix_mountPoint" ] ; then
      storage_add_posix_mountPoint_prefix=$(echo "$storage_add_posix_mountPoint" | head -c 1)
      if [ "$storage_add_posix_mountPoint_prefix" = "/" ] ; then
        break
      else
        message_imp "The path you provided is not an absolute path."
      fi
    else
      message_imp "There path that you specified is not a directory or does not exist."
    fi
  done
}

storage_interactive_ask_ceph() {
      storage_add_ceph_clusterName=$(ask_for_nonepty_string "$storage_add_ceph_clusterName" "Enter the Ceph cluster name (default: $storage_add_ceph_clusterName): ")
      storage_add_ceph_username=$(ask_for_nonepty_string "$storage_add_ceph_username" "Enter the Ceph username (default: $storage_add_ceph_username): ")
      storage_add_ceph_key=$(ask_for_nonepty_string "$storage_add_ceph_key" "Enter the Ceph key (default: $storage_add_ceph_key): ")
      storage_add_ceph_monitorHostname=$(ask_for_nonepty_string "$storage_add_ceph_monitorHostname" "Enter the hostname or IP of the Ceph monitor (default: $storage_add_ceph_monitorHostname): ")
      storage_add_ceph_poolName=$(ask_for_nonepty_string "$storage_add_ceph_poolName" "Enter the name of the Ceph pool to use (default: $storage_add_ceph_poolName): ")
      #storage_interactive_ask_insecure "Ceph"
}

storage_interactive_ask_gluster() {
      storage_add_gluster_hostname=$(ask_for_nonepty_string "$storage_add_gluster_hostname" "Enter the IP or hostname of a GlusterFS server (default: $storage_add_gluster_hostname): ")
      storage_add_gluster_volume=$(ask_for_nonepty_string "$storage_add_gluster_volume" "Enter the a name of a GlusterFS volume (default: $storage_add_gluster_volume): ")
      storage_add_gluster_transport=$(ask_for_nonepty_string "$storage_add_gluster_transport" "Does your GlusterFS use TCP or UDP? (valid answers: tcp/udp, default: $storage_add_gluster_transport): ")
      #storage_interactive_ask_insecure "GlusterFS"
}

storage_interactive_ask_s3() {
      storage_add_s3_hostname=$(ask_for_nonepty_string "$storage_add_s3_hostname" "Enter the IP or hostname of an S3 server (default: $storage_add_s3_hostname): ")
      storage_add_s3_bucketName=$(ask_for_nonepty_string "$storage_add_s3_bucketName" "Enter the a name of an S3 bucket (default: $storage_add_s3_bucketName): ")
      storage_add_s3_accessKey=$(ask_for_nonepty_string "$storage_add_s3_accessKey" "Enter the accessKey for the S3 server (default: $storage_add_s3_accessKey): ")
      storage_add_s3_secretKey=$(ask_for_nonepty_string "$storage_add_s3_secretKey" "Enter the secretKey for the S3 server (default: $storage_add_s3_secretKey): ")
      #storage_interactive_ask_insecure "S3"
}

storage_interactive_ask_swift() {
      storage_add_swift_authUrl=$(ask_for_nonepty_string "$storage_add_swift_authUrl" "Enter the IP or hostname of a GlusterFS server (default: $storage_add_swift_authUrl): ")
      storage_add_swift_tenantName=$(ask_for_nonepty_string "$storage_add_swift_tenantName" "Enter the IP or hostname of a GlusterFS server (default: $storage_add_swift_tenantName): ")
      storage_add_swift_containerName=$(ask_for_nonepty_string "$storage_add_swift_containerName" "Enter the IP or hostname of a GlusterFS server (default: $storage_add_swift_containerName): ")
      storage_add_swift_username=$(ask_for_nonepty_string "$storage_add_swift_username" "Enter the IP or hostname of a GlusterFS server (default: $storage_add_swift_username): ")
      storage_add_swift_password=$(ask_for_nonepty_string "$storage_add_swift_password" "Enter the IP or hostname of a GlusterFS server (default: $storage_add_swift_password): ")
      #storage_interactive_ask_insecure "Swift"
}

storage_interactive_ask_storage_add_read_only() {
  ######
  # Should this storage be set as read only when exposed?   
  ######
  if [ "$storage_add_read_only" = "true" ]; then
      storage_add_read_only_default=y
  else
      storage_add_read_only_default=n
  fi
  storage_add_read_only=""
  while [ "$storage_add_read_only" != "n" ] && [ "$storage_add_read_only" != "y" ] ; do
    _message "Expose storage as read only? (y/n, default: $storage_add_read_only_default): "
    read -r storage_add_read_only </dev/tty
    [  "$storage_add_read_only" = "" ] && storage_add_read_only=$storage_add_read_only_default
  done
  if [ "$storage_add_read_only" = "y" ]; then
      storage_add_read_only=true
  else
      storage_add_read_only=false
  fi
}

storage_interactive_ask_insecure() {
      storage_interactive_ask_insecure_label="$1"
      storage_add_insecure_default=y
      storage_add_insecure=""
      while [ "$storage_add_insecure" != "n" ] && [ "$storage_add_insecure" != "y" ] ; do
        _message "Does your $storage_interactive_ask_insecure_label server use https connection with a valid certificate? (y/n, default: $storage_add_insecure_default): "
        read -r storage_add_insecure </dev/tty
        [  "$storage_add_insecure" = "" ] && storage_add_insecure=$storage_add_insecure_default
      done
      if [ "$storage_add_insecure" = "n" ]; then
          storage_add_insecure=0
      else
          storage_add_insecure=1
      fi
}

storage_interactive_ask_luma_url() {
      storage_add_luma_url=$(ask_for_string "Please provide the url to the LUMA server, or leave it blank: ")
}

######
# Ask user for input and validate it's not empty
######
ask_for_nonepty_string() {
  current_value="$1"
  qestion="$2"

  default_value="$current_value"
  current_value=""
  while true ; do
    _message "$qestion"
    read -r current_value </dev/tty
    [  "$current_value" = "" ] && current_value="$default_value"
    [  "$current_value" != "" ] && break
  done
  echo "$current_value"
}

######
# Ask user for input and validate it's not empty
######
ask_for_string() {
  qestion="$1"
  _message "$qestion"
  read -r current_value </dev/tty
  echo "$current_value"
}

storage_add() {
  storage_add_json="$1"

  add_storage_script=$(cat <<EEOF
#!/usr/bin/env bash
cat <<EOF >/tmp/json
${storage_add_json}
EOF
cat /tmp/json | onepanel-rest-cli addStorage -
EEOF
  )

  echo "$add_storage_script" | $sh_c "docker run \
    -i \
    --network=host \
    -e TERM=xterm \
    -e ONEPANEL_HOST=\"https://localhost:9443\" \
    -e ONEPROVIDER_HOST=\"https://localhost\" \
    -e ONEPANEL_BASIC_AUTH=\"onepanel:$config_admin_password\" \
    --rm \
    --entrypoint bash \
    \"$rest_cli_docker_image\" -c \
    'echo \"-k\" > ~/.curlrc ;  cat > script.sh ; chmod +x script.sh ; ./script.sh'"
}

#
# storage add
# posix subcommand
#
storage_add_posix_main() {

  storage_add_storage_name="$1"
  shift
  storage_add_common_json="$1"
  shift
  storage_add_luma_json="$1"
  shift

  storage_add_posix_mountPoint=""
  while [ $(( $# )) -ne 0 ] ; do
      case $1 in
          -h|-\?|--help)   # Call a "usage" function to display a synopsis, then exit.
              storage_add_usage
              exit 0
              ;;
          -m|--mount-point)
              storage_add_posix_mountPoint="$2"
              shift
              ;;
          -?*|*)
              printf 'storage_add_posix_main WARN: Unknown option (ignored): %s\n' "$1" >&2
              exit 1
              ;;
      esac
      shift
  done

  # TODO validate mount point
  # has to start form the root /
  if [ "$storage_add_posix_mountPoint" = "" ]; then
    echo "Missing mandatory --mount-point parameter. Please see --help."
    exit 1
  fi

storage_add_json=$(cat <<EOF
  {
    "$storage_add_storage_name": {
      "type": "posix",
      "mountPoint": "/hostfs${storage_add_posix_mountPoint}"
$storage_add_common_json
$storage_add_luma_json
    }
  }
EOF
)

  # Since we are mounting / that is not needed anymore
  # # Check if there is a needed volume is compose
  # if ! grep -q "^[[:space:]]*\- \"$storage_add_posix_mountPoint:/onedatify$storage_add_posix_mountPoint\"" "$onedatify_docker_compose_path"; then
  #   # If the system is running ask for shutdown
  #   if onedatify_is_active; then
  #     message "Adding a posix storage to a provider that is running in a container requires"
  #     message "to stop the container, add $storage_add_posix_mountPoint as a volume and start provider again."
  #     if ! storage_add_ask_for_restart; then
  #       exit 1
  #     else
  #       onedatify_stop
  #     fi
  #   fi
  #   $sh_c "sed -i -e 's%volumes:%volumes:\\n\
  #      - \"${storage_add_posix_mountPoint}:/onedatify${storage_add_posix_mountPoint}\"%' \"$onedatify_docker_compose_path\""
  # fi    # Check if there is a needed volume is compose
  # #if storage_add_posix_was_oneprovider_running; then 

  . "${onedatify_config_dir}/install.sh"
  if [ "$storage_interactive_flag" = "1" ]; then
    install_and_run_oneprovider
  else
    onedatify_start
    install_provider_nagios_check
  fi


  storage_add "$storage_add_json"
}

#
# storage add
# s3 subcommand
#
storage_add_s3_main() {

  storage_add_storage_name="$1"
  shift
  storage_add_common_json="$1"
  shift
  storage_add_luma_json="$1"
  shift

  # mandatory
  storage_add_s3_hostname=""
  storage_add_s3_bucketName=""
  storage_add_s3_accessKey=""
  storage_add_s3_secretKey=""

  # optional
  storage_add_s3_signatureVersion=""

  while [ $(( $# )) -ne 0 ] ; do
      case $1 in
          -h|-\?|--help)   # Call a "usage" function to display a synopsis, then exit.
              storage_add_usage
              exit 0
              ;;
          -n|--hostname)
              storage_add_s3_hostname="$2"
              shift
              ;;
          -b|--bucket-name)
              storage_add_s3_bucketName="$2"
              shift
              ;;
          -a|--access-key)
              storage_add_s3_accessKey="$2"
              shift
              ;;
          -s|--secret-key)
              storage_add_s3_secretKey="$2"
              shift
              ;;
          -v|--signature-version)
              storage_add_s3_signatureVersion="$2"
              shift
              ;;
          -?*|*)
              printf 'storage_add_s3_main WARN: Unknown option (ignored): %s\n' "$1" >&2
              exit 1
              ;;
      esac
      shift
  done

  # TODO validate mount point
  if [ "$storage_add_s3_hostname" = "" ]; then
    echo "Missing mandatory --hostname parameter. Please see --help."
    exit 1
  fi

  # TODO validate mount point
  if [ "$storage_add_s3_bucketName" = "" ]; then
    echo "Missing mandatory --bucket-name parameter. Please see --help."
    exit 1
  fi

  # TODO validate mount point
  if [ "$storage_add_s3_accessKey" = "" ]; then
    echo "Missing mandatory --access-key parameter. Please see --help."
    exit 1
  fi

  # TODO validate mount point
  if [ "$storage_add_s3_secretKey" = "" ]; then
    echo "Missing mandatory --secret-key parameter. Please see --help."
    exit 1
  fi

  # TODO validate this
  # this is optional
  if [ "$storage_add_s3_signatureVersion" != "" ]; then
    storage_add_s3_signatureVersion_json=",\"signatureVersion\":\"$storage_add_s3_signatureVersion\""
    exit 1
  fi

storage_add_json=$(cat <<EOF
  {
    "$storage_add_storage_name": {
      "type": "s3",
      "hostname": "$storage_add_s3_hostname",
      "bucketName": "$storage_add_s3_bucketName",
      "accessKey": "$storage_add_s3_accessKey",
      "secretKey": "$storage_add_s3_secretKey"
$storage_add_s3_signatureVersion_json
$storage_add_common_json
$storage_add_luma_json
    }
  }
EOF
)

  #if storage_add_posix_was_oneprovider_running; then
  . "${onedatify_config_dir}/install.sh"
  if [ "$storage_interactive_flag" = "1" ]; then

    install_and_run_oneprovider
  else

    onedatify_start
    install_provider_nagios_check
  fi


  storage_add "$storage_add_json"
}

#
# storage add
# ceph subcommand
#
storage_add_ceph_main() {

  storage_add_storage_name="$1"
  shift
  storage_add_common_json="$1"
  shift
  storage_add_luma_json="$1"
  shift

  # mandatory
  storage_add_ceph_clusterName=""
  storage_add_ceph_username=""
  storage_add_ceph_key=""
  storage_add_ceph_monitorHostname=""
  storage_add_ceph_poolName=""

  while [ $(( $# )) -ne 0 ] ; do
      case $1 in
          -h|-\?|--help)   # Call a "usage" function to display a synopsis, then exit.
              storage_add_usage
              exit 0
              ;;
          -n|--cluster-name)
              storage_add_ceph_clusterName="$2"
              shift
              ;;
          -u|--username)
              storage_add_ceph_username="$2"
              shift
              ;;
          -k|--key)
              storage_add_ceph_key="$2"
              shift
              ;;
          -m|--monitor-hostname)
              storage_add_ceph_monitorHostname="$2"
              shift
              ;;
          -p|--pool-name)
              storage_add_ceph_poolName="$2"
              shift
              ;;
          -b|--block-size)
              storage_add_ceph_blockSize="$2"
              shift
              ;;
          -?*|*)
              printf 'storage_add_ceph_main WARN: Unknown option (ignored): %s\n' "$1" >&2
              exit 1
              ;;
      esac
      shift
  done

  # TODO validate mount point
  if [ "$storage_add_ceph_clusterName" = "" ]; then
    echo "Missing mandatory --cluster-name parameter. Please see --help."
    exit 1
  fi

  # TODO validate mount point
  if [ "$storage_add_ceph_username" = "" ]; then
    echo "Missing mandatory --username parameter. Please see --help."
    exit 1
  fi

  # TODO validate mount point
  if [ "$storage_add_ceph_key" = "" ]; then
    echo "Missing mandatory --key parameter. Please see --help."
    exit 1
  fi

  # TODO validate mount point
  if [ "$storage_add_ceph_monitorHostname" = "" ]; then
    echo "Missing mandatory --monitor-hostname parameter. Please see --help."
    exit 1
  fi

  # TODO validate mount point
  if [ "$storage_add_ceph_poolName" = "" ]; then
    echo "Missing mandatory --pool-name parameter. Please see --help."
    exit 1
  fi

  # TODO validate this
  # this is optional
  if [ "$storage_add_ceph_blockSize" != "" ]; then
    storage_add_ceph_blockSize_json=",\"blockSize\":\"$storage_add_ceph_blockSize\""
  fi


storage_add_json=$(cat <<EOF
  {
    "$storage_add_storage_name": {
      "type": "cephrados",
      "clusterName": "$storage_add_ceph_clusterName",
      "username": "$storage_add_ceph_username",
      "key": "$storage_add_ceph_key",
      "monitorHostname": "$storage_add_ceph_monitorHostname",
      "poolName": "$storage_add_ceph_poolName"
$storage_add_ceph_blockSize_json
$storage_add_common_json
$storage_add_luma_json
    }
  }
EOF
)


  # TODO throw error at some point that provider is down
  #if storage_add_posix_was_oneprovider_running; then
  . "${onedatify_config_dir}/install.sh"
  if [ "$storage_interactive_flag" = "1" ]; then
    install_and_run_oneprovider
  else
    onedatify_start
    install_provider_nagios_check
  fi

  storage_add "$storage_add_json"
}

#
# storage add
# gluster subcommand
#
storage_add_gluster_main() {

  storage_add_storage_name="$1"
  shift
  storage_add_common_json="$1"
  shift
  storage_add_luma_json="$1"
  shift

  # mandatory
  storage_add_gluster_hostname=""
  storage_add_gluster_volume=""

  # optional
  storage_add_gluster_transport=""
  storage_add_gluster_port=""
  storage_add_gluster_mountPoint=""
  storage_add_gluster_xlatorOptions

  while [ $(( $# )) -ne 0 ] ; do
      case $1 in
          -h|-\?|--help)   # Call a "usage" function to display a synopsis, then exit.
              storage_add_usage
              exit 0
              ;;
          -n|--hostname)
              storage_add_gluster_hostname="$2"
              shift
              ;;
          -v|--volume)
              storage_add_gluster_volume="$2"
              shift
              ;;
          -t|--transport-protocol)
              storage_add_gluster_transport="$2"
              shift
              ;;
          -p|--port)
              storage_add_gluster_port="$2"
              shift
              ;;
          -m|--mount-point)
              storage_add_gluster_mountPoint="$2"
              shift
              ;;
          -x|--xlator-options)
              storage_add_gluster_xlatorOptions="$2"
              shift
              ;;
          -?*|*)
              printf 'storage_add_gluster_main WARN: Unknown option (ignored): %s\n' "$1" >&2
              exit 1
              ;;
      esac
      shift
  done

  # TODO validate mount point
  if [ "$storage_add_gluster_hostname" = "" ]; then
    echo "Missing mandatory --hostname parameter. Please see --help."
    exit 1
  fi

  # TODO validate mount point
  if [ "$storage_add_gluster_volume" = "" ]; then
    echo "Missing mandatory --volume parameter. Please see --help."
    exit 1
  fi

  # TODO validate this
  # this is optional
  if [ "$storage_add_gluster_port" != "" ]; then
    storage_add_gluster_port_json=",\"port\":\"$storage_add_gluster_port\""
  fi

  # TODO validate this
  # this is optional
  if [ "$storage_add_gluster_transport" != "" ]; then
    storage_add_gluster_transport_json=",\"transport\":\"$storage_add_gluster_transport\""
  fi

  # TODO validate this
  # this is optional
  if [ "$storage_add_gluster_mountPoint" != "" ]; then
    storage_add_gluster_mountPoint_json=",\"mountPoint\":\"$storage_add_gluster_mountPoint\""
  fi

  # TODO validate this
  # this is optional
  if [ "$storage_add_gluster_xlatorOptions" != "" ]; then
    storage_add_gluster_xlatorOptions_json=",\"xlatorOptions\":\"$storage_add_gluster_xlatorOptions\""
  fi


storage_add_json=$(cat <<EOF
  {
    "$storage_add_storage_name": {
      "type": "glusterfs",
      "hostname": "$storage_add_gluster_hostname",
      "volume": "$storage_add_gluster_volume",
$storage_add_gluster_port_json
$storage_add_gluster_transport_json
$storage_add_gluster_mountPoint_json
$storage_add_gluster_xlatorOptions_json
$storage_add_common_json
$storage_add_luma_json
    }
  }
EOF
)

  #if storage_add_posix_was_oneprovider_running; then
  . "${onedatify_config_dir}/install.sh"
  if [ "$storage_interactive_flag" = "1" ]; then
    install_and_run_oneprovider
  else
    onedatify_start
    install_provider_nagios_check
  fi

  storage_add "$storage_add_json"
}

#
# storage add
# swift subcommand
#
storage_add_swift_main() {

  storage_add_storage_name="$1"
  shift
  storage_add_common_json="$1"
  shift
  storage_add_luma_json="$1"
  shift

  # mandatory
  storage_add_swift_authUrl=""
  storage_add_swift_tenantName=""
  storage_add_swift_containerName=""
  storage_add_swift_username=""
  storage_add_swift_password=""

  while [ $(( $# )) -ne 0 ] ; do
      case $1 in
          -h|-\?|--help)   # Call a "usage" function to display a synopsis, then exit.
              storage_add_usage
              exit 0
              ;;
          -a|--auth-url)
              storage_add_swift_authUrl="$2"
              shift
              ;;
          -t|--tenant-name)
              storage_add_swift_tenantName="$2"
              shift
              ;;
          -c|--container-name)
              storage_add_swift_containerName="$2"
              shift
              ;;
          -u|--username)
              storage_add_swift_username="$2"
              shift
              ;;
          -v|--password)
              storage_add_swift_password="$2"
              shift
              ;;
          -b|--block-size)
              storage_add_swift_blockSize="$2"
              shift
              ;;
          -?*|*)
              printf 'storage_add_swift_main WARN: Unknown option (ignored): %s\n' "$1" >&2
              exit 1
              ;;
      esac
      shift
  done

  # TODO validate mount point
  if [ "$storage_add_swift_authUrl" = "" ]; then
    echo "Missing mandatory --auth-url parameter. Please see --help."
    exit 1
  fi

  # TODO validate mount point
  if [ "$storage_add_swift_tenantName" = "" ]; then
    echo "Missing mandatory --tenant-name parameter. Please see --help."
    exit 1
  fi

  # TODO validate mount point
  if [ "$storage_add_swift_containerName" = "" ]; then
    echo "Missing mandatory --container-name parameter. Please see --help."
    exit 1
  fi

  # TODO validate mount point
  if [ "$storage_add_swift_username" = "" ]; then
    echo "Missing mandatory --username parameter. Please see --help."
    exit 1
  fi

  # TODO validate mount point
  if [ "$storage_add_swift_password" = "" ]; then
    echo "Missing mandatory --password parameter. Please see --help."
    exit 1
  fi

  # TODO validate this
  # this is optional
  if [ "$storage_add_swift_blockSize" != "" ]; then
    storage_add_swift_blockSize_json=",\"blockSize\":\"$storage_add_swift_blockSize\""
  fi

storage_add_json=$(cat <<EOF
  {
    "$storage_add_storage_name": {
      "type": "swift",
      "authUrl": "$storage_add_swift_authUrl",
      "tenantName": "$storage_add_swift_tenantName",
      "containerName": "$storage_add_swift_containerName",
      "username": "$storage_add_swift_username",
      "password": "$storage_add_swift_password"
$storage_add_swift_blockSize_json
$storage_add_common_json
$storage_add_luma_json
    }
  }
EOF
)

  #if storage_add_posix_was_oneprovider_running; then
  echo "$storage_interactive_flag"
  . "${onedatify_config_dir}/install.sh"
  if [ "$storage_interactive_flag" = "1" ]; then
    install_and_run_oneprovider
  else
    onedatify_start
    install_provider_nagios_check
  fi

  storage_add "$storage_add_json"
}
